/*

 Module that provides a LIB$INITIALIZE routine for the GNV toolset that
 will turn on some CRTL features that are not enabled by default.

 The CRTL features can also be turned on via logical names, but that
 impacts all programs and some aren't ready, willing, or able to handle
 the settings that GNV needs.

 Edit History

 1-001	John Reagan	Initial version using the old style interface
			but with the new version commented out.

 1-002  John Reagan	Switch to new API for setting features

 1-003	Steve Pitcher	Add DECC$RENAME_NO_INHERIT.

 1-004	Steve Pitcher	Quiet these, if the DECC feature doesn't exist.  

 1-005  Martin Borgman  Using the suggested UNIX level of 30
*/

#include <stdio.h>
#include <errno.h>

int   decc$feature_get_index (char *name);
int   decc$feature_set_value (int index, int mode, int value);

/*
** Sets current value for a feature
*/
static void set(char *name, int value)
{
     int index;
     errno = 0;

     index = decc$feature_get_index(name);

     if (index > 0)
	decc$feature_set_value(index, 1, value);
}

/*
** Sets default value for a feature
*/
static void set_default(char *name, int value)
{
     int index;
     errno = 0;

     index = decc$feature_get_index(name);

     if (index > 0)
	decc$feature_set_value (index, 0, value);
}

static void set_coe ( void )
{
    /* New style interface that works only on very recent 
       (Apr 2001 and beyond) CRTLs */

    /*
     * Only setting defaults allows logical names to 
     * override these settings.
     */

    /*   General Corrections          (DECC$UNIX_LEVEL == 1) */

     set_default ("DECC$FIXED_LENGTH_SEEK_TO_EOF", TRUE);
     set_default ("DECC$POSIX_SEEK_STREAM_FILE", TRUE);
     set_default ("DECC$SELECT_IGNORES_INVALID_FD", TRUE);
     set_default ("DECC$STRTOL_ERANGE", TRUE);
     set_default ("DECC$VALIDATE_SIGNAL_IN_KILL", TRUE);

    /*   General Enhancements        (DECC$UNIX_LEVEL == 10) */

     set_default ("DECC$ARGV_PARSE_STYLE", TRUE);
     set_default ("DECC$EFS_CASE_PRESERVE", TRUE);
     set_default ("DECC$STDIO_CTX_EOL", TRUE);
     set_default ("DECC$PIPE_BUFFER_SIZE", 8192);
     set_default ("DECC$USE_RAB64", TRUE);

    /*   UNIX-style file names       (DECC$UNIX_LEVEL == 20) */

     set_default ("DECC$DISABLE_TO_VMS_LOGNAME_TRANSLATION", TRUE);
     set_default ("DECC$EFS_CHARSET", TRUE);
     set_default ("DECC$FILENAME_UNIX_NO_VERSION", TRUE);
     set_default ("DECC$FILENAME_UNIX_REPORT", TRUE);
     set_default ("DECC$READDIR_DROPDOTNOTYPE", TRUE);
     set_default ("DECC$RENAME_NO_INHERIT", TRUE);
     set_default ("DECC$GLOB_UNIX_STYLE", TRUE);

    /*   UNIX-like file attributes     (DECC$UNIX_LEVEL == 30) */

     set_default ("DECC$EFS_FILE_TIMESTAMPS", TRUE);
     set_default ("DECC$EXEC_FILEATTR_INHERITANCE", TRUE);
     set_default ("DECC$FILE_OWNER_UNIX", TRUE);
     set_default ("DECC$FILE_PERMISSION_UNIX", TRUE);
     set_default ("DECC$FILE_SHARING", TRUE);

    /****

         UNIX-compliant behavior       (DECC$UNIX_LEVEL == 90)

     set_default ("DECC$FILENAME_UNIX_ONLY", TRUE);
     set_default ("DECC$POSIX_STYLE_UID", TRUE);
     set_default ("DECC$USE_JPI$_CREATOR", TRUE);
     set_default ("DECC$DETACHED_CHILD_PROCESS", TRUE);

    ****/
}

#pragma extern_model save
#pragma extern_model strict_refdef "LIB$INITIALIZE" nowrt, long
#if __INITIAL_POINTER_SIZE
#    pragma __pointer_size __save
#    pragma __pointer_size 32
#else
#    pragma __required_pointer_size __save
#    pragma __required_pointer_size 32
#endif
void (* const iniarray[])() = {set_coe, } ;	/* Set our contribution to the LIB$INITIALIZE array */
#if __INITIAL_POINTER_SIZE
#    pragma __pointer_size __restore
#else
#    pragma __required_pointer_size __restore
#endif
#pragma extern_model restore


/*
** Force a reference to LIB$INITIALIZE to ensure it
** exists in the image.
*/
int LIB$INITIALIZE();
globaldef int (*lib_init_ref)() = LIB$INITIALIZE;
